/* 
 * Copyright (C) 2005 Martin Koegler
 *    
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 * USA.
 */

#include <rfb/SSecurityFactoryTLS.h>
#include <rfb/SSecurityPlain.h>
#include <rfb/SSecurityTLS.h>
#include <rfb/SSecurityX509.h>
#include <rfb/SSecurityStack.h>
#include <rfb/Configuration.h>

using namespace rfb;

#ifdef HAVE_GNUTLS

StringParameter X509_CertFile
("x509cert",
 "specifies path to the x509 certificate in PEM format",
 "");

StringParameter X509_KeyFile
("x509key",
 "specifies path to the key of the x509 certificate in PEM format",
 "");

#endif

class Valid : public PasswordValidator {
public:
  bool validate(SConnection* sc, const char* username, const char* password)
  {
    if(*username==*password)
      return true;
    else
      return false;
  }
};
static Valid defvalid;

SSecurity* SSecurityFactoryTLS::getSSecurity(rdr::U8 secType, bool reverse) {
  switch (secType) {
#ifdef HAVE_GNUTLS
    /* Test:
  case secTypePlain:
    return new SSecurityPlain(&defvalid);
    */
  case secTypeTLSNone:
    return new SSecurityTLS();
  case secTypeTLSVnc:
    return new SSecurityStack(secTypeTLSVnc,getSSecurity(secTypeTLSNone,reverse),getSSecurity(secTypeVncAuth,reverse));
  case secTypeTLSPlain:
    return new SSecurityStack(secTypeTLSPlain,getSSecurity(secTypeTLSNone,reverse),getSSecurity(secTypePlain,reverse));
  case secTypeX509None:
    return new SSecurityX509(X509_CertFile.getData(),X509_KeyFile.getData());
  case secTypeX509Vnc:
    return new SSecurityStack(secTypeX509Vnc,getSSecurity(secTypeX509None,reverse),getSSecurity(secTypeVncAuth,reverse));
  case secTypeX509Plain:
    return new SSecurityStack(secTypeX509Vnc,getSSecurity(secTypeX509None,reverse),getSSecurity(secTypePlain,reverse));
#endif
  default:
    return SSecurityFactoryStandard::getSSecurity(secType,reverse);
  }
}
bool SSecurityFactoryTLS::isSecTypeSupported(rdr::U8 secType) {
  switch (secType) {
#ifdef HAVE_GNUTLS
  case secTypePlain:
  case secTypeTLSNone:
  case secTypeTLSVnc:
  case secTypeTLSPlain:
  case secTypeX509None:
  case secTypeX509Vnc:
  case secTypeX509Plain:
#endif
    return true;
  default:
    return SSecurityFactoryStandard::isSecTypeSupported(secType);
  }
}
